@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# Lint as: python3
 """Constructs inference version of the models.
 
 N.B. (jesseengel): I tried to make a nice base class. I tried both with multiple
@@ -156,6 +155,9 @@ class VSTBaseModule(models.Autoencoder):
     self.n_harmonics = output_splits['harmonic_distribution']
     self.n_noise = output_splits['noise_magnitudes']
 
+    # Get RNN dimesnions.
+    self.state_size = gin.query_parameter('RnnFcDecoder.rnn_channels')
+
     # Get interpolation method.
     self.resample_method = gin.query_parameter('Harmonic.amp_resample_method')
 
@@ -201,9 +203,10 @@ class VSTExtractFeatures(VSTBaseModule):
     """Parse the model operative config with special streaming parameters."""
     # Customize config.
     config = [
-        'OnlineF0PowerPreprocessor.time_steps = 1',
-        'OnlineF0PowerPreprocessor.center_power = False',
-        'OnlineF0PowerPreprocessor.center_f0 = False',
+        'OnlineF0PowerPreprocessor.padding = "valid"',
+        'OnlineF0PowerPreprocessor.compute_f0 = True',
+        'OnlineF0PowerPreprocessor.compute_power = True',
+        'OnlineF0PowerPreprocessor.viterbi = False',
     ]
     if self.crepe_saved_model_path is not None:
       config.append('OnlineF0PowerPreprocessor.crepe_saved_model_path = '
@@ -296,29 +299,122 @@ class VSTPredictControls(VSTBaseModule):
     return amps, hd, noise
 
 
+class VSTStatelessPredictControls(VSTBaseModule):
+  """Predict VST controls, but explicitly handle RNN state."""
+
+  def configure_gin(self):
+    """Parse the model operative config with special streaming parameters."""
+    config = [
+        'RnnFcDecoder.stateless = True',
+    ]
+    with gin.unlock_config():
+      gin.parse_config(config)
+
+  @property
+  def _signatures(self):
+    return {'call': self.call.get_concrete_function(
+        f0_scaled=tf.TensorSpec(shape=[1], dtype=tf.float32),
+        pw_scaled=tf.TensorSpec(shape=[1], dtype=tf.float32),
+        state=tf.TensorSpec(shape=[self.state_size], dtype=tf.float32),
+    )}
+
+  def build_network(self):
+    """Run a fake batch through the network."""
+    # Need two frames because of interpolation.
+    f0_scaled = tf.zeros([1])
+    pw_scaled = tf.zeros([1])
+    state = tf.zeros([self.state_size])
+    self._build_network(f0_scaled, pw_scaled, state)
+
+  @tf.function
+  def call(self, f0_scaled, pw_scaled, state):
+    """Convert f0 and loudness to synthesizer parameters."""
+    f0_scaled = tf.reshape(f0_scaled, [1, 1, 1])
+    pw_scaled = tf.reshape(pw_scaled, [1, 1, 1])
+    state = tf.reshape(state, [1, self.state_size])
+
+    f0_hz = ddsp.training.preprocessing.inv_scale_f0_hz(f0_scaled)
+
+    inputs = {
+        'f0_scaled': f0_scaled,
+        'pw_scaled': pw_scaled,
+        'state': state,
+    }
+
+    # Run through the model.
+    outputs = self.decoder(inputs, training=False)
+
+    # Apply the nonlinearities.
+    harm_controls = self.processor_group.harmonic.get_controls(
+        outputs['amps'], outputs['harmonic_distribution'], f0_hz)
+
+    noise_controls = self.processor_group.filtered_noise.get_controls(
+        outputs['noise_magnitudes']
+    )
+
+    # Return 1-D tensors.
+    amps = harm_controls['amplitudes'][0, 0]
+    hd = harm_controls['harmonic_distribution'][0, 0]
+    noise = noise_controls['magnitudes'][0, 0]
+    state = outputs['state'][0]
+    return amps, hd, noise, state
+
+
 @gin.configurable
-class VSTSynthesize(VSTBaseModule):
+class VSTSynthesize(tf.keras.Model):
   """VST inference modules, for models trained with `models/vst/vst.gin`."""
 
   def __init__(self,
                ckpt,
                new_hop_size=None,
-               verbose=True,
                **kwargs):
+    super().__init__(**kwargs)
     self.new_hop_size = new_hop_size
-    super().__init__(ckpt, **kwargs)
+    self.parse_gin(ckpt)
+    self.build_network()
 
-  def configure_gin(self):
+  # Carried over from VSTBaseModule. Need separate class to not include vars.
+  def save_model(self, save_dir):
+    """Saves a SavedModel after initialization."""
+    # self.save(save_dir)
+    tf.saved_model.save(self, save_dir, signatures=self._signatures)
+
+  def _build_network(self, *dummy_inputs):
+    """Helper function to build the network with dummy input args."""
+    print('Inputs to Model:', ddsp.core.map_shape(dummy_inputs))
+    unused_outputs = self(*dummy_inputs)
+    print('Outputs from Model:', ddsp.core.map_shape(unused_outputs))
+
+  def parse_gin(self, ckpt):
     """Parse the model operative config with special streaming parameters."""
-    # Customize config.
-    self.hop_size = self.new_hop_size if self.new_hop_size else self.hop_size
+    parse_operative_config(ckpt)
+
+    # Get Frame Size / Hop Size.
+    self.frame_size = gin.query_parameter('%frame_size')
+    frame_rate = gin.query_parameter('%frame_rate')
+    self.sample_rate = gin.query_parameter('%sample_rate')
+    self.hop_size = self.sample_rate // frame_rate
+
+    # Get number of outputs.
+    output_splits = dict(gin.query_parameter('RnnFcDecoder.output_splits'))
+    self.n_harmonics = output_splits['harmonic_distribution']
+    self.n_noise = output_splits['noise_magnitudes']
+
+    # Get interpolation method.
+    self.resample_method = gin.query_parameter('Harmonic.amp_resample_method')
+
     config = [
-        f'FilteredNoise.n_samples = {self.hop_size}',
         'harmonic_oscillator_bank.use_angular_cumsum = True',
     ]
     with gin.unlock_config():
       gin.parse_config(config)
 
+    self.hop_size = self.new_hop_size if self.new_hop_size else self.hop_size
+    self.filtered_noise = ddsp.synths.FilteredNoise(
+        n_samples=self.hop_size,
+        window_size=gin.query_parameter('FilteredNoise.window_size'),
+    )
+
   @property
   def _signatures(self):
     return {'call': self.call.get_concrete_function(
@@ -374,7 +470,7 @@ class VSTSynthesize(VSTBaseModule):
         sample_rate=self.sample_rate,
         amp_resample_method=self.resample_method)
 
-    noise_audio = self.processor_group.filtered_noise.get_signal(noise)
+    noise_audio = self.filtered_noise.get_signal(noise)
     audio_out = harm_audio + noise_audio
 
     # Return 1-D outputs.
