@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# Lint as: python3
 """Library of neural network functions."""
 
 import inspect
@@ -32,6 +31,21 @@ tfkl = tfk.layers
 # pylint: disable=redundant-keyword-arg
 
 
+def gin_register_keras_layers():
+  """Registers all keras layers and Sequential to be referenceable in gin."""
+  # Register sequential model.
+  gin.external_configurable(tf.keras.Sequential, 'tf.keras.Sequential')
+
+  # Register all the layers.
+  for k, v in inspect.getmembers(tf.keras.layers):
+    # Duck typing for tf.keras.layers.Layer since keras uses metaclasses.
+    if hasattr(v, 'variables'):
+      gin.external_configurable(v, f'tf.keras.layers.{k}')
+
+
+gin_register_keras_layers()
+
+
 class DictLayer(tfkl.Layer):
   """Wrap a Keras Layer to take dictionary inputs and outputs.
 
@@ -856,7 +870,8 @@ class Rnn(tfkl.Layer):
   def __init__(self, dims, rnn_type, return_sequences=True, bidir=False,
                **kwargs):
     super().__init__(**kwargs)
-    rnn_class = {'lstm': tfkl.LSTM, 'gru': tfkl.GRU}[rnn_type]
+    rnn_class = {'lstm': tfkl.LSTM,
+                 'gru': tfkl.GRU}[rnn_type]
     self.rnn = rnn_class(dims, return_sequences=return_sequences)
     if bidir:
       self.rnn = tfkl.Bidirectional(self.rnn)
@@ -865,6 +880,31 @@ class Rnn(tfkl.Layer):
     return self.rnn(x)
 
 
+@gin.register
+class StatelessRnn(tfkl.Layer):
+  """Stateless unidirectional RNN for streaming models."""
+
+  def __init__(self, dims, rnn_type, **kwargs):
+    super().__init__(**kwargs)
+    rnn_class = {'lstm': tfkl.LSTM,
+                 'gru': tfkl.GRU}[rnn_type]
+    self.rnn = rnn_class(dims, return_sequences=True, return_state=True)
+
+  def call(self, x, state):
+    """Make a call with explicit carrying of state.
+
+    Args:
+      x: Input, shape [batch, T, dims_in].
+      state: Last output, shape [batch, dims].
+
+    Returns:
+      y: Output, shape [batch, T, dims].
+      new_state: Carried state, shape [batch, dims]
+    """
+    y, new_state = self.rnn(x, initial_state=state)
+    return y, new_state
+
+
 @gin.register
 class RnnFc(tfk.Sequential):
   """RNN layer -> fully connected -> LayerNorm -> Activation fn."""
